@@ -8,7 +8,7 @@
 
     use constant NS_SOAP_ENV => "http://schemas.xmlsoap.org/soap/envelope/";
 
-    our $VERSION = '0.3';
+    our $VERSION = '0.4';
 
     __PACKAGE__->mk_accessors qw(wsdlobj decoders encoders);
 
@@ -74,6 +74,14 @@
 
         return $self->NEXT::end($c, @_) unless $soap;
 
+        if (scalar @{$c->error}) {
+            $c->stash->{soap}->fault
+              ({ code => [ 'env:Sender' ],
+                 reason => 'Unexpected Error', detail =>
+                 'Unexpected error in the application: '.(join "\n", @{$c->error} ).'!'});
+            $c->error(0);
+        }
+
         my $namespace = $soap->namespace || NS_SOAP_ENV;
         my $response = XML::LibXML->createDocument();
 
